% **********************************************************************************
% ***** Berechnung und Darstellung der Flugbahn eines rotierenden Tennisballes *****
% **********************************************************************************
%
% Das Programm berechnet die Flugbahn eines Tennisballes in Vakuum, in Luft, 
% in Luft mit Topspin und in Luft mit Slice. Die vier Kurven werden in ein
% Fenster gezeichnet, wobei sowohl das Fenster wie auch der Bildname frei 
% gewaehlt werden koennen. Zusaetzlich werden in einem Ausgabefenster die 
% Flugzeiten angegeben. Die Berechnung kann beliebig oft wiederholt werden
% es koennen also beliebig viele moevies erzeugt werden.
% Ausserdem kann man jederzeit in einem frei zu waehlenden Fenster ein 
% bereits erzeugtes movie betrachten.
% 
%
% Im folgenden werden zuerst die Positionen der fuenfzehn Fenster, die
% prinzipiell geoeffnet werden koennten, festgelegt.  Die Variable fuer
% die Nummer des Fensters heisst fignr. 
% Bilder ist ein Text, der zuerst nur "Berechnung durchfuehren enthaelt
% Bei jedem weiteren Berechnungsdurchlauf wird der Name des Bildes, der
% eingegeben wird, dazugehaengt. (Siehe letzte Zeile im "Kapitel" Graph.)
% Das heisst, in der Menuleiste wird nach jedem Berechnungsdurchgang ein
% neuer Button mit dem Bildnamen hinzugefuegt.
% Die Variable num zaehlt die Anzahl der berechneten Bilder und wird nur
% in der Eingabe zur Bestimmung der Wortlaenge gebraucht.
%
% Die Variable index gibt wieder, welcher Button der Menuleiste gewaehlt
% wurde. Ist index=1, so wurde "exit" gewaehlt, das Programm ist beendet.
% Ist index>1 wird das Programm durchlaufen, wobei Fallunterscheidungen
% getroffen werden muessen. Fuer index=2 wurde das Feld "Figure No. (max 15)"
% gewaehlt. Das heisst, das zu oeffnende Fenster wird festgelegt. 
% Fuer index=3 wurde das Fenster "Berechnung durchfuehren" gewaehlt.
% Das Programm wird durchlaufen, ein neues movie erzeugt.
% Ist der Index jedoch >3, so wurde ein bereits erzeugtes movie gewaehlt.
% Jenes soll ablaufen.  
%
clear all;
for k=1:3:13
    pos(k,:)=[586 434 430 281];
    pos(k+1,:)=[586 95 430 281];
    pos(k+2,:)=[142 95 430 281];
end    
num=1;
index=2;
fignr=1;
Bilder=',''Berechnung durchfuehren''';
while index>1
    strg=['index=menu(''Bitte waehle aus'',''exit'',''Figure No.' num2str(fignr) ' (max 15)''' Bilder ');'];
    eval(strg);

    if index==2
        fignr=input('Figure No. ');
    end
     

    if index==3
        % *****************
        % **** EINGABE ****
        % *****************
        %
        % Die Eingabe erfolgt im DECterm. Waehlt man die ENTER-Taste, wird der  
        % in runde Klammern angegebene Standartwert verwendet. Es ist physikalisch 
        % nicht sinnvoll, eine andere Dichte einzugeben, da die 
        % Wechselwirkungskoeffizienten jene von Luft sind.
        % Die Rechenzeit kann frei gewaehlt werden, da nur die Flugdauer des Balles 
        % in Vakuum analytisch berechnet werden kann.  Die Flugdauer eines Balles
        % mit Slice liegt aber zumeist ueber dieser.
        %
        % In der Variablen xin steht die Anfangsbedingung der DGL 
        % (Ort und Geschwindigkeit in 2 Koordinaten).
        % Die Variable tmaxid entspricht der Flugzeit des Balles in Vakuum,
        % sie wurde analytisch berechnet.
        %

        moviename=input('Name des Bildes: ','s');
        movienamelaenge(num)=length(moviename);
        num=num+1;
        global g alpha w etha
        g=9.81 ; 
        rho=input('Dichte des Mediums (Luft:1.29 kg/m^3) = ');
        if isempty(rho) 
            rho=1.29;
        end
        m=input('Masse des Balles (0.05) kg = ');
        if isempty(m) 
            m=0.05;
        end
        d=input('Durchmesser des Balles (0.063) m = ');
        if isempty(d) 
           d=0.063;
        end
        alpha=pi*d^2/(8*m)*rho;
        w=input('Umfangsgeschwindigkeit des Balles (20) m/s = ');
        if isempty(w) 
            w=20;
        end
        h=input('Hoehe (1) m = ');
        if isempty(h) 
            h=1;
        end
        v0=input('Anfangsgeschwindigkeit [13.6->28 (25)]  m/s = ');
        if isempty(v0) 
            v0=25;
        end
        theta=pi/180*input('Abschusswinkel [0->90 (15)] grad) = ');
        if isempty(theta) 
            theta=pi/180*15;
        end
        step=input('Zeitaufloesung (0.01) Sekundenschritte = ');
        if isempty(step) 
            step=0.01;
        end
        xin=[0,h,v0*cos(theta),v0*sin(theta)]
        tmaxid=(xin(4)+sqrt(xin(4)^2+2*g*xin(2)))/g
        rechenzeit=input('maximale Flugzeit (Vakuumzeit) s = ');
        if isempty(rechenzeit) 
            rechenzeit=tmaxid;
        end
        

        % ************************
        % **** LOESEN DER DGL ****
        % ************************
        %
        % Zuerst werden die Optionenen fuer die DGL gesetzt.
        % Dann wird die Loesung fuer Vakuum berechnet und
        % der Matrix [tid,xid] uebergeben.
        % Analog werden die Loesungen fuer Luft ([t0,x0])
        % fuer Luft mit Topspin (Etha=1) ([t1,x1])
        % fuer Luft mit Slice (Etha=-1) ([t2,x2])
        % berechnet und den entsprechenden Matrizen
        % uebergeben.
        % Abschliessend wird die Berechnung der 
        % maximalen Achsenlaenge durchgefuehrt, da alle vier Graphen
        % in ein Bild gezeichnet werden.
        %

        options=odeset('Events','on','Maxstep',tmaxid/100);
        [tid,xid]=ode23('tennisip2',[0 rechenzeit],xin,options);
        [t0,x0]=ode23('tennis0p2',[0 rechenzeit],xin,options);
        etha=1;
        [t1,x1]=ode23('tennis1p2',[0 rechenzeit],xin,options);
        etha=-1;
        [t2,x2]=ode23('tennis1p2',[0 rechenzeit],xin,options);

        xmax=max([max(xid(:,1)) max(x0(:,1)) max(x1(:,1)) max(x2(:,1))]);
        ymax=max([max(xid(:,2)) max(x0(:,2)) max(x1(:,2)) max(x2(:,2))]);
        

        % *******************************
        % **** AUSGABE DER FLUGDAUER ****
        % *******************************
        %
        % Ausgabe im DECterm
             % Flugdauer in Vakuum
        tmaxid=(xin(4)+sqrt(xin(4)^2+2*g*xin(2)))/g;
             % Flugdauer in Luft
        tmax0=t0(length(t0))
             % Flugdauer in Luft mit Topspin
        tmax1=t1(length(t1))
             % Flugdauer in Luft mit Slice
        tmax2=t2(length(t2))
        
        % Ausgabe im Ausgabefenster
        % Die Ausgabedaten werden unter AusgabeBildname abgespeichert,
        % um sicher zu gehen, dass unter Bildname kein Befehl verwendet
        % wurde.
        %
        figure(16);
        set(16,'Position',[256 440 305 278]);
        clf;
        axis([0 5 0 5]);
        axis off;
        text(0.5,5,'Flugdauer des Tennisballes');
        text(0.5,4,[moviename]);
        text(-0.5,3,['in Vakuum: ' num2str(tmaxid) ' Sekunden']);
        text(-0.5,2,['in Luft: ' num2str(tmax0) ' Sekunden']);
        text(-0.5,1,['in Luft mit Topspin: ' num2str(tmax1) ' Sekunden']);
        text(-0.5,0,['in Luft mit Slice: ' num2str(tmax2) ' Sekunden']);
        Ausgabe=getframe(gcf);
        strg=['Ausgabe' moviename '=Ausgabe;'];
        eval(strg);
       

        % *********************************
        % **** Interpolation der Daten ****
        % *********************************
        %
        % Da das numerische Loesungsverfahren beliebige Datenpunkte liefert, 
        % muessen die einzelnen Graphen interpoliert werden.
        % zeitmax enthaelt die maximale Flugzeit (meist slice!)
        % timemax bewirkt, dass nur solange, als notwendig gezeichnet wird
        % (fuer den Fall, dass die gewaehlte Rechenzeit groesser als alle
        %  Flugzeiten ist.).
        % step ist die gewaehlte Schrittweite. Sie muss zu timemax addiert
        % werden, um zu sichern, dass im Falle einer Abrundung  von timemax
        % tatsaechlich bis zum Ende von timemax gezeichnet wird.
        %
        zeitmax=max([tmaxid tmax0 tmax1 tmax2]);
        timemax=min([zeitmax rechenzeit]);
        time=0:step:(timemax+step);
        xids=spline(tid,xid(:,1),time);
        yids=spline(tid,xid(:,2),time);
        x0s=spline(t0,x0(:,1),time);
        y0s=spline(t0,x0(:,2),time);
        x1s=spline(t1,x1(:,1),time);
        y1s=spline(t1,x1(:,2),time);
        x2s=spline(t2,x2(:,1),time);
        y2s=spline(t2,x2(:,2),time);
        

        % ***************
        % **** Graph ****
        % ***************
        %
        % Im folgenden wird der Graph erzeugt. Beachtenswert ist die
        % mitlaufende Zeit. Sie wurde zuerst umgerechnet, um in
        % "Echtzeit" mit dem Graphen mitzulaufen. Die kleine Zahl wurde
        % addiert, um in der Anzeige keinen Sprung zu erzeugen, falls
        % die Sekunde eine ganze Zahl ist.
        % Die Achsen wurden vergroessert, um das Bild nicht vollstaendig
        % auszufuellen.
        %
        figure(fignr);
        set(fignr,'Position',pos(fignr,:));
        clf;
        axis([0,xmax*1.13,0,ymax*1.13]);
        hold on;
        xlabel('Weite in Meter');
        ylabel('Hoehe in Meter');
        p=plot(0,h,'EraseMode','none','Color','g');
        p=plot(0,h,'EraseMode','none','Color','b');
        p=plot(0,h,'EraseMode','none','Color','r');
        p=plot(0,h,'EraseMode','none','Color','c');
        legend('Vakuum','Luft','Topspin','Slice');
        M=moviein(length(xids),gcf);
        M(:,1)=getframe(gcf);
        for j=2:length(xids)
            x=xids([j-1:j]);
            y=yids([j-1:j]);
            set(p,'XData',x,'YData',y,'Color','g')
            drawnow
            x=x0s([j-1:j]);
            y=y0s([j-1:j]);
            set(p,'XData',x,'YData',y,'Color','b')
            drawnow
            x=x1s([j-1:j]);
            y=y1s([j-1:j]);
            set(p,'XData',x,'YData',y,'Color','r')
            drawnow
            x=x2s([j-1:j]);
            y=y2s([j-1:j]);
            set(p,'XData',x,'YData',y,'Color','c')
            drawnow
            zeit=[num2str(j*timemax/length(xids)+0.000001,15)];
            title([moviename ': Zeit = ' zeit(1:4) ' Sekunden']);  
            M(:,j)=getframe(gcf);
        end
        strg=['M' moviename '=M;'];
        eval(strg);
        help=Bilder;
        Bilder=[help ',''' moviename ''''];
    end     % von if index==3
    
    
    if index>3
        Ende=26;
        for hilfs=1:(index-3)
            Ende=Ende+3+movienamelaenge(hilfs);
        end
        Anfang=Ende-movienamelaenge(index-3);
        Ende=Ende-1;
        moviename=Bilder(Anfang:Ende);
  
        figure(16);
        clf;
        strg=['Ausgabe=' 'Ausgabe' moviename ';'];
        eval(strg);
        movie(gcf,Ausgabe,1)

        figure(fignr);
        set(fignr,'Position',pos(fignr,:));
        clf;
        strg=['M=' 'M' moviename ';'];
        eval(strg);
        movie(gcf,M,1)
    end   % von if index>3
end  %von while
